import * as i0 from '@angular/core';
import { InjectionToken, EventEmitter, Directive, Inject, Input, Output, HostListener, Optional, ContentChildren, NgModule } from '@angular/core';
import { asyncScheduler, Subject, BehaviorSubject, Subscription, merge, ReplaySubject, combineLatest } from 'rxjs';
import { throttleTime, switchMap, map, startWith, first } from 'rxjs/operators';
import { Validators, NG_VALUE_ACCESSOR } from '@angular/forms';
import { CommonModule } from '@angular/common';
import { circularDeepEqual } from 'fast-equals';
import { __awaiter } from 'tslib';

const CONFIG_TOKEN = new InjectionToken('NGRX_CLEAN_FORMS_CONFIG');
const defaultConfig = {
    throttleTime: 20,
    distinctWritesOnly: true,
};
const throttle = (config) => throttleTime(config.throttleTime, asyncScheduler, {
    leading: true,
    trailing: true,
});

function isFormArray(formArray) {
    return typeof formArray === 'object' && Array.isArray(formArray.controls);
}
function isFormGroup(formGroup) {
    return (typeof formGroup === 'object' &&
        formGroup.controls &&
        typeof formGroup.controls === 'object');
}

/**
 * Maps an object of control states to a value given by the `mapFunc`.
 *
 * @param controls Object which contains keys and associated control update.
 * @param mapFunc Mapper function to convert the update.
 *
 * @example
 * // Mapping of:
 * const controls = {
 *  firstControl: {
 *    value: 'firstValue',
 *    ...
 *  },
 *  secondControl: {
 *    value: 'secondValue',
 *    ...
 *  },
 * }
 *
 * // With the function:
 * const mapFunc = (control) => control.value
 *
 * // Will result in:
 * const result = {
 *  firstControl: 'firstValue',
 *  secondControl: 'secondValue'
 * }
 */
function mapFormGroupControlStates(controls, mapFunc) {
    return mapFormControls(controls, mapFunc);
}
/**
 * Maps an object of control summaries to a value given by the `mapFunc`.
 *
 * @param controls Object which contains keys and associated control summaries.
 * @param mapFunc Mapper function to convert the update.
 *
 * @example
 * // Mapping of:
 * const controls = {
 *  firstControl: {
 *    value: 'firstValue',
 *    ...
 *  },
 *  secondControl: {
 *    value: 'secondValue',
 *    ...
 *  },
 * }
 *
 * // With the function:
 * const mapFunc = (control) => control.value
 *
 * // Will result in:
 * const result = {
 *  firstControl: 'firstValue',
 *  secondControl: 'secondValue'
 * }
 */
function mapFormGroupControlSummaries(controls, mapFunc) {
    return mapFormControls(controls, mapFunc);
}
/**
 * Maps an object of control updates to a value given by the `mapFunc`.
 *
 * @param controls Object which contains keys and associated control updates.
 * @param mapFunc Mapper function to convert the update.
 *
 * @example
 * // Mapping of:
 * const controls = {
 *  firstControl: {
 *    value: 'firstValue',
 *    ...
 *  },
 *  secondControl: {
 *    value: 'secondValue',
 *    ...
 *  },
 * }
 *
 * // With the function:
 * const mapFunc = (control) => control.value
 *
 * // Will result in:
 * const result = {
 *  firstControl: 'firstValue',
 *  secondControl: 'secondValue'
 * }
 */
function mapFormGroupControlUpdates(controls, mapFunc) {
    return mapFormControls(controls, mapFunc);
}
function mapFormControls(controls, mapFunc) {
    const result = Object.entries(controls)
        .map(([key, control]) => ({
        [key]: mapFunc(control, key),
    }))
        .reduce((ctrl1, ctrl2) => (Object.assign(Object.assign({}, ctrl1), ctrl2)), {});
    return result;
}
/**
 * Converts one or multiple Angular validators for the usage with this framework.
 * Does **not** support asynchronous validators.
 * Uses Validators.compose from @angular/forms.
 *
 * Only a subset of properties is available for the Angular validators:
 * `value, dirty, pristine, touched, untouched, enabled, disabled`
 * @param fn Angular validator function..
 */
function validatorOf(...fn) {
    const composed = Validators.compose(fn);
    return (control) => composed({
        value: control.value,
        dirty: !control.pristine,
        pristine: control.pristine,
        touched: !control.untouched,
        untouched: control.untouched,
        enabled: !control.disabled,
        disabled: control.disabled,
    });
}
/**
 * Merges an array of `FormControlErrors`.
 * Returns null if all errors are null.
 * @param errors An array of `FormControlErrors` which will be merged into a single `FormControlErrors` value.
 */
function mergeFormControlErrors(...errors) {
    return errors.reduce((e1, e2) => {
        if (!e1 && !e2) {
            return null;
        }
        if (!e1) {
            return e2;
        }
        if (!e2) {
            return e1;
        }
        return Object.assign(Object.assign({}, e1), e2);
    }, null);
}
/**
 * Merges an array of errors.
 * The different child control errors will also be merged.
 * Returns null if all errors are null.
 * @param errors Multiple errors to be merged.
 *
 * @example
 * // Merge of:
 * {
 *   stringControl: {
 *     firstError: 'firstError'
 *   }
 * },
 * {
 *   stringControl: {
 *     secondError: 'secondError'
 *   }
 * }
 *
 * // Will result in:
 * {
 *   stringControl: {
 *     firstError: 'firstError',
 *     secondError: 'secondError'
 *   }
 * }
 *
 */
function mergeFormGroupErrors(...errors) {
    return errors.reduce((group1, group2) => {
        if (!group1 && !group2) {
            return null;
        }
        if (!group1) {
            return group2;
        }
        if (!group2) {
            return group1;
        }
        return Object.assign(Object.assign(Object.assign({}, group1), group2), Object.keys(group1)
            .filter(key1 => Object.keys(group2).find(key2 => key1 === key2))
            .map(key => ({
            [key]: mergeFormControlErrors(group1[key], group2[key]),
        }))
            .reduce((e1, e2) => (Object.assign(Object.assign({}, e1), e2)), {}));
    }, null);
}
/**
 * Merges an array of errors.
 * Errors at each index will also be merged.
 * Returns null if all errors are null.
 * @param errors Multiple errors to be merged.
 *
 * @example
 * // Merge of:
 * [
 *   {
 *     firstError: 'firstError'
 *   }
 * ],
 * [
 *   stringControl: {
 *     secondError: 'secondError'
 *   }
 * ]
 *
 * // Will result in:
 * [
 *   {
 *     firstError: 'firstError',
 *     secondError: 'secondError'
 *   }
 * ]
 *
 */
function mergeFormArrayErrors(...errors) {
    const mergedErrors = errors
        .filter(Boolean)
        .reduce((arr1, arr2) => (arr1.length >= arr2.length ? arr1 : arr2), [])
        .map((_, i) => mergeFormControlErrors(...errors.map(arrayErrors => arrayErrors && arrayErrors[i])));
    return mergedErrors.filter(Boolean).length > 0 ? mergedErrors : null;
}

/**
 * Returns a `FormControlState` which represents the input state with the applied update.
 * The returned value is a new constructed object, except when the update is invalid.
 *
 * @param control The `FormControlState` which the update will be applied to.
 * @param update The `FormControlUpdate` itself.
 */
function reduceFormControl(control, update) {
    if (!update) {
        return control;
    }
    return Object.assign(Object.assign({}, control), update);
}
/**
 * Returns a `FormGroupState` which represents the input state with the applied update.
 * The returned value is a new constructed object, except when the update is invalid.
 *
 * @param control The `FormGroupState` which the update will be applied to.
 * @param update The `FormGroupUpdate` itself.
 */
function reduceFormGroup(group, update) {
    if (!update) {
        return group;
    }
    return Object.assign(Object.assign(Object.assign({}, group), update), { controls: Object.assign(Object.assign({}, group.controls), (update.controls
            ? mapFormGroupControlUpdates(update.controls, (control, key) => (Object.assign(Object.assign({}, group.controls[key]), control)))
            : {})) });
}
/**
 * Returns a `FormArrayState` which represents the input state with the applied update.
 * The returned value is a new constructed object, except when the update is invalid.
 *
 * @param control The `FormArrayState` which the update will be applied to.
 * @param update The `FormArrayUpdate` itself. Invalid values and empty objects inside the `update.controls` array will be ignored.
 */
function reduceFormArray(group, update) {
    if (!update) {
        return group;
    }
    return Object.assign(Object.assign(Object.assign({}, group), update), { controls: group.controls.map((control, index) => update && update.controls ? reduceFormControl(control, update.controls[index]) : control) });
}

/**
 * Initializes a new `FormControlState`.
 * @see `FormControlInit`
 *
 * @example
 * initFormControl(['value', [validator]])
 * initFormControl({value: 'value', validators: [validator], disabled: true, ...})
 */
function initFormControl(initial) {
    return Array.isArray(initial)
        ? initFormControlFromTuple(initial)
        : initFormControlFromUpdate(initial);
}
/**
 * Initializes a new `FormGroupState`.
 * @param controls An object containing control initializations.
 * @param validators An array of GroupValidators which will be used for additional error detection.
 * @see `FormGroupInit`, `GroupValidator`
 *
 * @example
 * initFormGroup({
 *  firstControl: ['firstValue', [validator]],
 *  secondControl: {value: 'value', validators: [validator], disabled: true, ...}
 * })
 */
function initFormGroup(controls, validators = []) {
    return {
        controls: Object.entries(controls)
            .map(([key, initial]) => ({
            [key]: initFormControl(initial),
        }))
            .reduce((ctrl1, ctrl2) => (Object.assign(Object.assign({}, ctrl1), ctrl2)), {}),
        validators,
    };
}
/**
 * Initializes a new `FormArrayState`.
 * @param controls An object containing control initializations.
 * @param validators An array of ArrayValidators which will be used for additional error detection.
 * @see `FormArrayInit`, `ArrayValidator`
 *
 * @example
 * initFormArray(
 *  [
 *      ['firstValue', [validator]],
 *      {value: 'value', validators: [validator], disabled: true, ...}
 *  ],
 *  [arrayValidator])
 */
function initFormArray(initial, validators = []) {
    return {
        controls: initial.map(init => initFormControl(init)),
        validators,
    };
}
function initFormControlFromTuple([value, validators = [], disabled = false,]) {
    return initFormControlFromUpdate({ value, validators, disabled });
}
function initFormControlFromUpdate(initialUpdate) {
    return reduceFormControl({
        value: undefined,
        initialValue: initialUpdate.value,
        pristine: true,
        untouched: true,
        disabled: false,
        validators: [],
    }, initialUpdate);
}

const cssClasses = {
    invalid: 'ng-invalid',
    valid: 'ng-valid',
    pristine: 'ng-pristine',
    dirty: 'ng-dirty',
    touched: 'ng-touched',
    untouched: 'ng-untouched',
    changed: 'ng-changed',
    initial: 'ng-initial',
};
const CONTROL_DIRECTIVE_SELECTOR = `ngrxFormControl`;
class AbstractControlDirective {
    constructor(ref, r2, injectedConfig) {
        this.ref = ref;
        this.r2 = r2;
        this.injectedConfig = injectedConfig;
        this.controlUpdate = new EventEmitter(true);
        this._controlSummary$ = new Subject();
        this.controlSummary$ = this._controlSummary$.asObservable();
        this.touched$ = new Subject();
        this.value$ = new Subject();
        this.config$ = new BehaviorSubject(this.injectedConfig);
        this.summarySubscription = new Subscription();
        this.config$
            .pipe(switchMap((config) => merge(this.touched$.pipe(throttle(config)), this.value$.pipe(throttle(config)))))
            .subscribe((value) => this.controlUpdate.emit(value));
    }
    set setControlSummary(controlSummary) {
        if (!controlSummary) {
            return;
        }
        this.updateSummary(controlSummary);
    }
    set setConfig(inputConfig) {
        const config = Object.assign(Object.assign({}, this.injectedConfig), inputConfig);
        this.config$.next(config);
    }
    get controlSummary() {
        return this._controlSummary;
    }
    isValueChanged(nextValue) {
        const disabled = !this.config$.value.distinctWritesOnly;
        const lastValue = this.getValue();
        return disabled || !circularDeepEqual(nextValue, lastValue);
    }
    emitTouched() {
        this.touched$.next({ untouched: false });
    }
    emitValue(value) {
        this.value$.next({ value, pristine: false });
    }
    updateSummary(summary) {
        this._controlSummary = summary;
        this._controlSummary$.next(summary);
        if (this.isValueChanged(summary.value)) {
            this.setValue(summary.value);
        }
        this.setDisabled(summary.disabled);
        this.chooseClass(cssClasses.invalid, cssClasses.valid, summary.valid);
        this.chooseClass(cssClasses.dirty, cssClasses.pristine, summary.pristine);
        this.chooseClass(cssClasses.touched, cssClasses.untouched, summary.untouched);
        this.chooseClass(cssClasses.initial, cssClasses.changed, summary.changed);
    }
    chooseClass(class1, class2, chooseSecond) {
        if (chooseSecond) {
            this.r2.addClass(this.ref.nativeElement, class2);
            this.r2.removeClass(this.ref.nativeElement, class1);
        }
        else {
            this.r2.addClass(this.ref.nativeElement, class1);
            this.r2.removeClass(this.ref.nativeElement, class2);
        }
    }
    ngOnDestroy() {
        this.summarySubscription.unsubscribe();
        this.config$.complete();
        this.touched$.complete();
        this.value$.complete();
        this._controlSummary$.complete();
    }
}
AbstractControlDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.1", ngImport: i0, type: AbstractControlDirective, deps: [{ token: i0.ElementRef }, { token: i0.Renderer2 }, { token: CONFIG_TOKEN }], target: i0.ɵɵFactoryTarget.Directive });
AbstractControlDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.0.1", type: AbstractControlDirective, inputs: { controlKey: ["ngrxFormControl", "controlKey"], setControlSummary: ["controlSummary", "setControlSummary"], setConfig: ["controlConfig", "setConfig"] }, outputs: { controlUpdate: "controlUpdate" }, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.1", ngImport: i0, type: AbstractControlDirective, decorators: [{
            type: Directive
        }], ctorParameters: function () { return [{ type: i0.ElementRef }, { type: i0.Renderer2 }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [CONFIG_TOKEN]
                }] }]; }, propDecorators: { controlKey: [{
                type: Input,
                args: [CONTROL_DIRECTIVE_SELECTOR]
            }], setControlSummary: [{
                type: Input,
                args: ['controlSummary']
            }], setConfig: [{
                type: Input,
                args: ['controlConfig']
            }], controlUpdate: [{
                type: Output
            }] } });

class RangeInputControlDirective extends AbstractControlDirective {
    constructor(ref, r2, config) {
        super(ref, r2, config);
    }
    onInput() {
        this.emitValue(this.getValue());
    }
    onBlur() {
        this.emitTouched();
    }
    getValue() {
        const value = this.ref.nativeElement.value;
        return value === '' ? null : parseFloat(value);
    }
    setValue(value) {
        this.r2.setProperty(this.ref.nativeElement, 'value', value);
    }
    setDisabled(disabled) {
        this.r2.setProperty(this.ref.nativeElement, 'disabled', disabled);
    }
}
RangeInputControlDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.1", ngImport: i0, type: RangeInputControlDirective, deps: [{ token: i0.ElementRef }, { token: i0.Renderer2 }, { token: CONFIG_TOKEN }], target: i0.ɵɵFactoryTarget.Directive });
RangeInputControlDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.0.1", type: RangeInputControlDirective, selector: "input[type=\"range\"][ngrxFormControl]", host: { listeners: { "input": "onInput()", "blur": "onBlur()" } }, providers: [
        {
            provide: AbstractControlDirective,
            useExisting: RangeInputControlDirective,
        },
    ], usesInheritance: true, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.1", ngImport: i0, type: RangeInputControlDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: `input[type="range"][${CONTROL_DIRECTIVE_SELECTOR}]`,
                    providers: [
                        {
                            provide: AbstractControlDirective,
                            useExisting: RangeInputControlDirective,
                        },
                    ],
                }]
        }], ctorParameters: function () { return [{ type: i0.ElementRef }, { type: i0.Renderer2 }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [CONFIG_TOKEN]
                }] }]; }, propDecorators: { onInput: [{
                type: HostListener,
                args: ['input']
            }], onBlur: [{
                type: HostListener,
                args: ['blur']
            }] } });

const RadioControlNotSupported = Error(
// tslint:disable-next-line: max-line-length
`Radio inputs are not yet supported. If you need to use one consider wrapping it in a Component which implements the ControlValueAccessor interface.`);
class RadioInputControlDirective extends AbstractControlDirective {
    constructor(ref, r2, config) {
        super(ref, r2, config);
        throw RadioControlNotSupported;
    }
    setValue(value) { }
    getValue() {
        return null;
    }
}
RadioInputControlDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.1", ngImport: i0, type: RadioInputControlDirective, deps: [{ token: i0.ElementRef }, { token: i0.Renderer2 }, { token: CONFIG_TOKEN }], target: i0.ɵɵFactoryTarget.Directive });
RadioInputControlDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.0.1", type: RadioInputControlDirective, selector: "input[type=\"radio\"][ngrxFormControl]", providers: [
        {
            provide: AbstractControlDirective,
            useExisting: RadioInputControlDirective,
        },
    ], usesInheritance: true, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.1", ngImport: i0, type: RadioInputControlDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: `input[type="radio"][${CONTROL_DIRECTIVE_SELECTOR}]`,
                    providers: [
                        {
                            provide: AbstractControlDirective,
                            useExisting: RadioInputControlDirective,
                        },
                    ],
                }]
        }], ctorParameters: function () { return [{ type: i0.ElementRef }, { type: i0.Renderer2 }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [CONFIG_TOKEN]
                }] }]; } });

const SelectControlNotSupported = Error(
// tslint:disable-next-line: max-line-length
`Select inputs are not yet supported. If you need to use one consider wrapping it in a Component which implements the ControlValueAccessor interface.`);
class SelectInputControlDirective extends AbstractControlDirective {
    constructor(ref, r2, config) {
        super(ref, r2, config);
        throw SelectControlNotSupported;
    }
    setValue(value) { }
    getValue() {
        return null;
    }
}
SelectInputControlDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.1", ngImport: i0, type: SelectInputControlDirective, deps: [{ token: i0.ElementRef }, { token: i0.Renderer2 }, { token: CONFIG_TOKEN }], target: i0.ɵɵFactoryTarget.Directive });
SelectInputControlDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.0.1", type: SelectInputControlDirective, selector: "select[ngrxFormControl]", providers: [
        {
            provide: AbstractControlDirective,
            useExisting: SelectInputControlDirective,
        },
    ], usesInheritance: true, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.1", ngImport: i0, type: SelectInputControlDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: `select[${CONTROL_DIRECTIVE_SELECTOR}]`,
                    providers: [
                        {
                            provide: AbstractControlDirective,
                            useExisting: SelectInputControlDirective,
                        },
                    ],
                }]
        }], ctorParameters: function () { return [{ type: i0.ElementRef }, { type: i0.Renderer2 }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [CONFIG_TOKEN]
                }] }]; } });

class NumberInputControlDirective extends AbstractControlDirective {
    constructor(ref, r2, config) {
        super(ref, r2, config);
    }
    onInput() {
        this.emitValue(this.getValue());
    }
    onBlur() {
        this.emitTouched();
    }
    getValue() {
        const value = this.ref.nativeElement.value;
        return value === '' ? null : parseFloat(value);
    }
    setValue(value) {
        this.r2.setProperty(this.ref.nativeElement, 'value', value === null ? '' : value);
    }
    setDisabled(disabled) {
        this.r2.setProperty(this.ref.nativeElement, 'disabled', disabled);
    }
}
NumberInputControlDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.1", ngImport: i0, type: NumberInputControlDirective, deps: [{ token: i0.ElementRef }, { token: i0.Renderer2 }, { token: CONFIG_TOKEN }], target: i0.ɵɵFactoryTarget.Directive });
NumberInputControlDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.0.1", type: NumberInputControlDirective, selector: "input[type=\"number\"][ngrxFormControl]", host: { listeners: { "input": "onInput()", "blur": "onBlur()" } }, providers: [
        {
            provide: AbstractControlDirective,
            useExisting: NumberInputControlDirective,
        },
    ], usesInheritance: true, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.1", ngImport: i0, type: NumberInputControlDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: `input[type="number"][${CONTROL_DIRECTIVE_SELECTOR}]`,
                    providers: [
                        {
                            provide: AbstractControlDirective,
                            useExisting: NumberInputControlDirective,
                        },
                    ],
                }]
        }], ctorParameters: function () { return [{ type: i0.ElementRef }, { type: i0.Renderer2 }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [CONFIG_TOKEN]
                }] }]; }, propDecorators: { onInput: [{
                type: HostListener,
                args: ['input']
            }], onBlur: [{
                type: HostListener,
                args: ['blur']
            }] } });

class TextInputControlDirective extends AbstractControlDirective {
    constructor(ref, r2, config) {
        super(ref, r2, config);
    }
    onInput() {
        this.emitValue(this.getValue());
    }
    onBlur() {
        this.emitTouched();
    }
    getValue() {
        return this.ref.nativeElement.value;
    }
    setValue(value) {
        this.r2.setProperty(this.ref.nativeElement, 'value', value);
    }
    setDisabled(disabled) {
        this.r2.setProperty(this.ref.nativeElement, 'disabled', disabled);
    }
}
TextInputControlDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.1", ngImport: i0, type: TextInputControlDirective, deps: [{ token: i0.ElementRef }, { token: i0.Renderer2 }, { token: CONFIG_TOKEN }], target: i0.ɵɵFactoryTarget.Directive });
TextInputControlDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.0.1", type: TextInputControlDirective, selector: "input[type=\"text\"][ngrxFormControl],textarea[ngrxFormControl],input:not([type=\"checkbox\"]):not([type=\"number\"]):not([type=\"range\"]):not([type=\"radio\"])[ngrxFormControl]", host: { listeners: { "input": "onInput()", "blur": "onBlur()" } }, providers: [
        {
            provide: AbstractControlDirective,
            useExisting: TextInputControlDirective,
        },
    ], usesInheritance: true, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.1", ngImport: i0, type: TextInputControlDirective, decorators: [{
            type: Directive,
            args: [{
                    // tslint:disable-next-line: max-line-length
                    selector: `input[type="text"][${CONTROL_DIRECTIVE_SELECTOR}],textarea[${CONTROL_DIRECTIVE_SELECTOR}],input:not([type="checkbox"]):not([type="number"]):not([type="range"]):not([type="radio"])[${CONTROL_DIRECTIVE_SELECTOR}]`,
                    providers: [
                        {
                            provide: AbstractControlDirective,
                            useExisting: TextInputControlDirective,
                        },
                    ],
                }]
        }], ctorParameters: function () { return [{ type: i0.ElementRef }, { type: i0.Renderer2 }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [CONFIG_TOKEN]
                }] }]; }, propDecorators: { onInput: [{
                type: HostListener,
                args: ['input']
            }], onBlur: [{
                type: HostListener,
                args: ['blur']
            }] } });

class ValueAccessorConnectorDirective extends AbstractControlDirective {
    constructor(ref, r2, config, accessors) {
        super(ref, r2, config);
        this.initAccessor(accessors);
    }
    initAccessor(accessors) {
        if (!accessors || !accessors.length) {
            return;
        }
        this.accessor = accessors[0];
        this.accessor.registerOnChange((val) => super.emitValue(val));
        this.accessor.registerOnTouched(() => super.emitTouched());
    }
    getValue() {
        return this.lastValue;
    }
    setValue(value) {
        if (!this.accessor) {
            return;
        }
        this.accessor.writeValue(value);
        this.lastValue = value;
    }
    setDisabled(disabled) {
        if (this.accessor && this.accessor.setDisabledState) {
            this.accessor.setDisabledState(disabled);
        }
    }
}
ValueAccessorConnectorDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.1", ngImport: i0, type: ValueAccessorConnectorDirective, deps: [{ token: i0.ElementRef }, { token: i0.Renderer2 }, { token: CONFIG_TOKEN }, { token: NG_VALUE_ACCESSOR, optional: true }], target: i0.ɵɵFactoryTarget.Directive });
ValueAccessorConnectorDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.0.1", type: ValueAccessorConnectorDirective, selector: "[ngrxFormControl]:not(input):not(textarea):not(select)", providers: [
        {
            provide: AbstractControlDirective,
            useExisting: ValueAccessorConnectorDirective,
        },
    ], usesInheritance: true, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.1", ngImport: i0, type: ValueAccessorConnectorDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: `[${CONTROL_DIRECTIVE_SELECTOR}]:not(input):not(textarea):not(select)`,
                    providers: [
                        {
                            provide: AbstractControlDirective,
                            useExisting: ValueAccessorConnectorDirective,
                        },
                    ],
                }]
        }], ctorParameters: function () { return [{ type: i0.ElementRef }, { type: i0.Renderer2 }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [CONFIG_TOKEN]
                }] }, { type: undefined, decorators: [{
                    type: Optional
                }, {
                    type: Inject,
                    args: [NG_VALUE_ACCESSOR]
                }] }]; } });

class CheckboxInputControlDirective extends AbstractControlDirective {
    constructor(ref, r2, config) {
        super(ref, r2, config);
    }
    onInput() {
        this.emitValue(this.getValue());
    }
    onBlur() {
        this.emitTouched();
    }
    getValue() {
        return this.ref.nativeElement.checked;
    }
    setValue(value) {
        this.r2.setProperty(this.ref.nativeElement, 'checked', value);
    }
    setDisabled(disabled) {
        this.r2.setProperty(this.ref.nativeElement, 'disabled', disabled);
    }
}
CheckboxInputControlDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.1", ngImport: i0, type: CheckboxInputControlDirective, deps: [{ token: i0.ElementRef }, { token: i0.Renderer2 }, { token: CONFIG_TOKEN }], target: i0.ɵɵFactoryTarget.Directive });
CheckboxInputControlDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.0.1", type: CheckboxInputControlDirective, selector: "input[type=\"checkbox\"][ngrxFormControl]", host: { listeners: { "input": "onInput()", "blur": "onBlur()" } }, providers: [
        {
            provide: AbstractControlDirective,
            useExisting: CheckboxInputControlDirective,
        },
    ], usesInheritance: true, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.1", ngImport: i0, type: CheckboxInputControlDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: `input[type="checkbox"][${CONTROL_DIRECTIVE_SELECTOR}]`,
                    providers: [
                        {
                            provide: AbstractControlDirective,
                            useExisting: CheckboxInputControlDirective,
                        },
                    ],
                }]
        }], ctorParameters: function () { return [{ type: i0.ElementRef }, { type: i0.Renderer2 }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [CONFIG_TOKEN]
                }] }]; }, propDecorators: { onInput: [{
                type: HostListener,
                args: ['input']
            }], onBlur: [{
                type: HostListener,
                args: ['blur']
            }] } });

const controlDirectives = [
    TextInputControlDirective,
    CheckboxInputControlDirective,
    NumberInputControlDirective,
    RadioInputControlDirective,
    RangeInputControlDirective,
    SelectInputControlDirective,
    ValueAccessorConnectorDirective,
];

class ControlChildrenDirective {
    getChildren() {
        return this.convertToObservable(this.abstractInputs);
    }
    convertToObservable(query) {
        return query.changes.pipe(map(() => query), startWith(query), map((list) => list.toArray()));
    }
}
ControlChildrenDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.1", ngImport: i0, type: ControlChildrenDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive });
ControlChildrenDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.0.1", type: ControlChildrenDirective, queries: [{ propertyName: "abstractInputs", predicate: AbstractControlDirective, descendants: true }], ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.1", ngImport: i0, type: ControlChildrenDirective, decorators: [{
            type: Directive
        }], propDecorators: { abstractInputs: [{
                type: ContentChildren,
                args: [AbstractControlDirective, { descendants: true }]
            }] } });

const UnkownControlError = (controlKey) => new Error(`The control "${controlKey}" is not part of the form.`);
class AbstractFormDirective extends ControlChildrenDirective {
    constructor(cdr) {
        super();
        this.cdr = cdr;
        this.formUpdate = new EventEmitter();
        this.formSummary$ = new ReplaySubject(1);
    }
    set setFormSummary(summary) {
        if (!summary) {
            return;
        }
        this.formSummary$.next(summary);
    }
    ngAfterViewInit() {
        const children$ = this.getChildren();
        children$
            .pipe(map((children) => children.map((child) => child.controlUpdate.pipe(map((update) => [update, child.controlKey])))), switchMap((children) => merge(...children)))
            .subscribe(([update, key]) => this.emitUpdate(update, key));
        combineLatest([children$, this.formSummary$]).subscribe(([children, summary]) => this.updateChildren(children, summary));
    }
    ngOnDestroy() {
        this.formSummary$.complete();
    }
    updateChildren(children, summary, shouldRetry = true) {
        return __awaiter(this, void 0, void 0, function* () {
            const safeChildren = children.filter(Boolean);
            const unknownControl = safeChildren.find((child) => summary.controls[child.controlKey] === undefined);
            if (unknownControl && !shouldRetry) {
                throw UnkownControlError(unknownControl.controlKey);
            }
            if (unknownControl) {
                this.cdr.detectChanges();
                const updatedChildren = yield this.getChildren().pipe(first()).toPromise();
                return this.updateChildren(updatedChildren, summary, false);
            }
            safeChildren.forEach((control) => {
                control.updateSummary(summary.controls[control.controlKey]);
            });
        });
    }
}
AbstractFormDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.1", ngImport: i0, type: AbstractFormDirective, deps: [{ token: i0.ChangeDetectorRef }], target: i0.ɵɵFactoryTarget.Directive });
AbstractFormDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.0.1", type: AbstractFormDirective, inputs: { setFormSummary: ["formSummary", "setFormSummary"] }, outputs: { formUpdate: "formUpdate" }, usesInheritance: true, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.1", ngImport: i0, type: AbstractFormDirective, decorators: [{
            type: Directive
        }], ctorParameters: function () { return [{ type: i0.ChangeDetectorRef }]; }, propDecorators: { setFormSummary: [{
                type: Input,
                args: ['formSummary']
            }], formUpdate: [{
                type: Output
            }] } });

class FormGroupDirective extends AbstractFormDirective {
    emitUpdate(update, key) {
        this.formUpdate.next({ controls: { [key]: update } });
    }
}
FormGroupDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.1", ngImport: i0, type: FormGroupDirective, deps: null, target: i0.ɵɵFactoryTarget.Directive });
FormGroupDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.0.1", type: FormGroupDirective, selector: "[ngrxFormGroup]", usesInheritance: true, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.1", ngImport: i0, type: FormGroupDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[ngrxFormGroup]',
                }]
        }] });

class FormArrayDirective extends AbstractFormDirective {
    emitUpdate(update, key) {
        const index = parseInt(key.toString(), 10);
        const controls = new Array(index + 1);
        controls[index] = update;
        this.formUpdate.next({ controls });
    }
}
FormArrayDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.1", ngImport: i0, type: FormArrayDirective, deps: null, target: i0.ɵɵFactoryTarget.Directive });
FormArrayDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.0.1", type: FormArrayDirective, selector: "[ngrxFormArray]", usesInheritance: true, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.1", ngImport: i0, type: FormArrayDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[ngrxFormArray]',
                }]
        }] });

/**
 * This module represents the entry point of ngrx-clean-forms.
 * It provides all necessary directives for the usage inside templates.
 *
 * The `.withConfig()` method can be used to pass a config to the module.
 *
 * - [npm](https://www.npmjs.com/package/ngrx-clean-forms)
 * - [documentation](https://github.com/NiklasPor/ngrx-clean-forms)
 */
class NgrxCleanFormsModule {
    /**
     * Creates a new module with the given config. Missing values will be set with default values.
     *
     * @param config A partial `FormsConfig` object, supplying config paramters.
     *
     * @example
     * NgrxCleanFormsModule.withConfig({
     *   throttleTime: 15,
     * }),
     *
     */
    static withConfig(config) {
        return {
            ngModule: NgrxCleanFormsModule,
            providers: [
                {
                    provide: CONFIG_TOKEN,
                    useValue: Object.assign(Object.assign({}, defaultConfig), config),
                },
            ],
        };
    }
}
NgrxCleanFormsModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.1", ngImport: i0, type: NgrxCleanFormsModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
NgrxCleanFormsModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "12.0.1", ngImport: i0, type: NgrxCleanFormsModule, declarations: [FormGroupDirective, FormArrayDirective, TextInputControlDirective, CheckboxInputControlDirective, NumberInputControlDirective, RadioInputControlDirective, RangeInputControlDirective, SelectInputControlDirective, ValueAccessorConnectorDirective], imports: [CommonModule], exports: [FormGroupDirective, FormArrayDirective, TextInputControlDirective, CheckboxInputControlDirective, NumberInputControlDirective, RadioInputControlDirective, RangeInputControlDirective, SelectInputControlDirective, ValueAccessorConnectorDirective] });
NgrxCleanFormsModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "12.0.1", ngImport: i0, type: NgrxCleanFormsModule, providers: [
        {
            provide: CONFIG_TOKEN,
            useValue: defaultConfig,
        },
    ], imports: [[CommonModule]] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.1", ngImport: i0, type: NgrxCleanFormsModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [FormGroupDirective, FormArrayDirective, ...controlDirectives],
                    imports: [CommonModule],
                    exports: [FormGroupDirective, FormArrayDirective, ...controlDirectives],
                    providers: [
                        {
                            provide: CONFIG_TOKEN,
                            useValue: defaultConfig,
                        },
                    ],
                }]
        }] });

/**
 * Resets a control back to the default values.
 * The following values will be ignored and stay the same:
 * - `initialValue`
 * - `validators`
 * - `disabled`
 *
 * @param control The `FormControlState` which should be used to create the reset.
 * @param initialValue Optional parameter for passing a new initial value.
 */
function resetFormControl(control, initialValue = control.initialValue) {
    return initFormControl([initialValue, control.validators, control.disabled]);
}
/**
 * Resets a group back to its default values.
 * Internally calls `resetFormControl` for every control.
 *
 * @see `resetFormControl`
 *
 * @param group The `FormGroupState` which should be used to create the reset.
 */
function resetFormGroup(group) {
    const controls = mapFormGroupControlStates(group.controls, control => resetFormControl(control));
    return reduceFormGroup(group, { controls });
}
/**
 * Resets an array back to its default values.
 * Internally calls `resetFormControl` for every control.
 *
 * @see `resetFormControl`
 *
 * @param array The `FormArrayState` which should be used to create the reset.
 */
function resetFormArray(array) {
    const controls = array.controls.map(control => resetFormControl(control));
    return reduceFormArray(array, { controls });
}

function getFormControlErrors(control) {
    return control.validators.map(validator => validator(control));
}
function getFormGroupControlSummariesErrors(summaries) {
    const errors = mapFormGroupControlSummaries(summaries, summary => summary.errors);
    Object.entries(errors)
        .filter(([_, value]) => value === null)
        .forEach(([key, _]) => delete errors[key]);
    return Object.keys(errors).length ? errors : null;
}
function getFormArrayControlSummariesErrors(summaries) {
    const errors = summaries.map(summary => summary.errors);
    return errors.filter(Boolean).length ? errors : null;
}
function getFormGroupErrors(group) {
    return mergeFormGroupErrors(...group.validators.map(validator => validator(group)));
}
function getFormArrayErrors(array) {
    return mergeFormArrayErrors(...array.validators.map(validator => validator(array)));
}
function getFormControlChanged(control) {
    return !circularDeepEqual(control.value, control.initialValue);
}
/**
 * Creates a `FormControlSummary` from the given `FormControlState`.
 * It is possible to add additional errors.
 *
 * @param control The `FormControlState` which is used to create the `FormControlSummary`.
 * @param additionalErrors An array of `FormControlErrors` which will be merged into the errors of the control.
 */
function getFormControlSummary(control, ...additionalErrors) {
    const errors = mergeFormControlErrors(...getFormControlErrors(control), ...additionalErrors);
    return Object.assign(Object.assign({}, control), { errors, valid: errors === null, changed: getFormControlChanged(control) });
}
function getFormGroupControlSummaries(controls, ...additionalErrors) {
    const additionalError = mergeFormGroupErrors(...additionalErrors);
    return mapFormGroupControlStates(controls, (control, key) => getFormControlSummary(control, additionalError ? additionalError[key] : null));
}
function getFormArrayControlSummaries(controls, ...additionalErrors) {
    const additionalError = mergeFormArrayErrors(...additionalErrors);
    return controls.map((control, i) => getFormControlSummary(control, additionalError ? additionalError[i] : null));
}
function getFormGroupPristine(group) {
    return Object.values(group.controls).every(control => control.pristine);
}
function getFormArrayPristine(array) {
    return array.controls.every(control => control.pristine);
}
function getFormGroupUntouched(group) {
    return Object.values(group.controls).every(control => control.untouched);
}
function getFormGroupChanged(controls) {
    return Object.values(controls).some(control => control.changed);
}
function getFormArrayChanged(controls) {
    return controls.some(control => control.changed);
}
function getFormArrayUntouched(array) {
    return array.controls.every(control => control.untouched);
}
function getFormArrayKeys(array) {
    return array.controls.map((_, i) => i);
}
/**
 * Creates a `FormGroupSummary` from the given `FormGroupState`.
 * It is possible to add additional errors.
 *
 * @param group The input `FormGroupState`. Used to create the `FormGroupSummary`.
 * @param additionalErrors An array of additional `FormGroupErrors`, which will be merged into the errors of the group and of each control.
 */
function getFormGroupSummary(group, ...additionalErrors) {
    const groupErrors = getFormGroupErrors(group);
    const summaries = getFormGroupControlSummaries(group.controls, groupErrors, ...additionalErrors);
    const errors = getFormGroupControlSummariesErrors(summaries);
    return {
        controls: summaries,
        pristine: getFormGroupPristine(group),
        untouched: getFormGroupUntouched(group),
        errors,
        valid: errors === null,
        changed: getFormGroupChanged(summaries),
        validators: group.validators,
    };
}
/**
 * Creates a `FormArraySummary` from the given `FormArrayState`.
 * It is possible to add additional errors.
 *
 * @param array The input `FormArrayState`. Used to create the `FormArraySummary`.
 * @param additionalErrors An array of additional `FormArrayErrors`, which will be merged into the errors of the array and of each control.
 */
function getFormArraySummary(array, ...additionalErrors) {
    const arrayErrors = getFormArrayErrors(array);
    const summaries = getFormArrayControlSummaries(array.controls, arrayErrors, ...additionalErrors);
    const errors = getFormArrayControlSummariesErrors(summaries);
    return {
        controls: summaries,
        keys: getFormArrayKeys(array),
        pristine: getFormArrayPristine(array),
        untouched: getFormArrayUntouched(array),
        errors,
        valid: errors === null,
        changed: getFormArrayChanged(summaries),
        validators: array.validators,
    };
}

/**
 * Generated bundle index. Do not edit.
 */

export { AbstractControlDirective, AbstractFormDirective, CONFIG_TOKEN, CONTROL_DIRECTIVE_SELECTOR, CheckboxInputControlDirective, ControlChildrenDirective, FormArrayDirective, FormGroupDirective, NgrxCleanFormsModule, NumberInputControlDirective, RadioControlNotSupported, RadioInputControlDirective, RangeInputControlDirective, SelectControlNotSupported, SelectInputControlDirective, TextInputControlDirective, UnkownControlError, ValueAccessorConnectorDirective, controlDirectives, defaultConfig, getFormArrayChanged, getFormArrayControlSummaries, getFormArrayControlSummariesErrors, getFormArrayErrors, getFormArrayKeys, getFormArrayPristine, getFormArraySummary, getFormArrayUntouched, getFormControlChanged, getFormControlErrors, getFormControlSummary, getFormGroupChanged, getFormGroupControlSummaries, getFormGroupControlSummariesErrors, getFormGroupErrors, getFormGroupPristine, getFormGroupSummary, getFormGroupUntouched, initFormArray, initFormControl, initFormGroup, isFormArray, isFormGroup, mapFormControls, mapFormGroupControlStates, mapFormGroupControlSummaries, mapFormGroupControlUpdates, mergeFormArrayErrors, mergeFormControlErrors, mergeFormGroupErrors, reduceFormArray, reduceFormControl, reduceFormGroup, resetFormArray, resetFormControl, resetFormGroup, throttle, validatorOf };
//# sourceMappingURL=ngrx-clean-forms.js.map
